#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 16 19:59:07 2021

@author: Alexander Southan
"""

import numpy as np
import unittest

from src.little_helpers import num_derive


class TestNumDerive(unittest.TestCase):

    def test_num_derive(self):
        x = list(range(50))
        y = []
        for curr_x in x:
            y.append(curr_x**2 + 0.31*curr_x - 3.2)
        y = [y]

        deriv_1 = num_derive.derivative(x, y, order=1, averaging_window=1)
        deriv_2 = num_derive.derivative(x, y, order=2, averaging_window=5)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 16 19:59:07 2021

@author: Alexander Southan
"""

import numpy as np
import statsmodels.api as sm
import unittest

from src.little_helpers import statsmodel_wrapper


class TestSatsmodelWrapper(unittest.TestCase):

    def test_statsmodel_wrapper(self):
        x = np.arange(50)
        y = 3*x

        a = statsmodel_wrapper(sm.OLS)
        a.fit(x, y)
        pred = a.predict(x)

        for curr_y, curr_pred in zip(y, pred):
            self.assertAlmostEqual(curr_y, curr_pred, 5)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 16 19:59:07 2021

@author: Alexander Southan
"""

import numpy as np
import unittest

from src.little_helpers import array_tools


class TestArrayTools(unittest.TestCase):

    def test_array_tools(self):

        # check for robustness to unsorted list input
        x_list = [5, 8, 9, 88, 56, 22]
        y_list = [33, 99, 111, 2222, 8, 1]
        idx = array_tools.closest_index(68, x_list)
        y_at_x = array_tools.y_at_x(7, x_list, y_list)

        segs = array_tools.segment_xy_values(x_list, [23, 86])

        self.assertEqual(idx, 4)
        self.assertEqual(y_at_x, 99)
        self.assertEqual(len(segs), 3)
        self.assertEqual(len(segs[0]), 4)
        self.assertEqual(len(segs[1]), 3)
        self.assertEqual(len(segs[2]), 1)

        # test with ascending ndarray input
        sort_idx = np.argsort(x_list)
        x_array = np.array(x_list)[sort_idx]
        y_array = np.array(y_list)[sort_idx]

        idx = array_tools.closest_index(68, x_array)
        y_at_x = array_tools.y_at_x(7, x_array, y_array)

        segs = array_tools.segment_xy_values(x_array, [23, 86])

        self.assertEqual(idx, 4)
        self.assertEqual(y_at_x, 99)
        self.assertEqual(len(segs), 3)
        self.assertEqual(len(segs[0]), 4)
        self.assertEqual(len(segs[1]), 3)
        self.assertEqual(len(segs[2]), 1)

        # test with descending ndarray input
        x_array = np.array(x_list)[sort_idx[::-1]]
        y_array = np.array(y_list)[sort_idx[::-1]]

        segs = array_tools.segment_xy_values(x_array, [23, 86])

        self.assertEqual(len(segs), 3)
        self.assertEqual(len(segs[0]), 4)
        self.assertEqual(len(segs[1]), 3)
        self.assertEqual(len(segs[2]), 1)

        # test segmentation with x and y values
        segs_x, segs_y = array_tools.segment_xy_values(
            x_array, [23, 86], y_values=y_array)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 16 19:59:07 2021

@author: Alexander Southan
"""

import numpy as np
import unittest

from src.little_helpers import hydrogel_conversions


class TestHydrogelConversions(unittest.TestCase):

    def test_hydrogel_conversions(self):

        # test the different arguments
        eds_values = [1, 2, 3, 4]

        v_2s_1 = hydrogel_conversions.eds_to_volume_fraction(
            eds_values, eds_mode='plain', output='polymer')
        v_2s_2 = hydrogel_conversions.eds_to_volume_fraction(
            eds_values, eds_mode='subtracted', output='polymer')

        self.assertEqual((v_2s_2.sum()), np.array([0.5, 1/3, 0.25, 0.2]).sum())
        self.assertEqual((v_2s_1.sum()), np.array([1, 0.5, 1/3, 0.25]).sum())

        v_2s_3 = hydrogel_conversions.eds_to_volume_fraction(
            eds_values, eds_mode='plain', output='solvent')
        v_2s_4 = hydrogel_conversions.eds_to_volume_fraction(
            eds_values, eds_mode='subtracted', output='solvent')

        self.assertEqual((v_2s_4.sum()),
                         (1-np.array([0.5, 1/3, 0.25, 0.2])).sum())
        self.assertEqual((v_2s_3.sum()),
                         (1-np.array([1, 0.5, 1/3, 0.25])).sum())

        # test error messages
        self.assertRaises(
            ValueError, hydrogel_conversions.eds_to_volume_fraction,
            eds_values, 1, 1, 'subtacted', 'polymer')
        self.assertRaises(
            ValueError, hydrogel_conversions.eds_to_volume_fraction,
            eds_values, 1, 1, 'subtracted', 'polmer')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 16 19:59:07 2021

@author: Alexander Southan
"""

import numpy as np
import unittest

from src.little_helpers import math_functions


class TestMathFunctions(unittest.TestCase):

    def test_math_functions(self):

        c_e = [1, 2, 3, 4]
        q_m = 3
        K_s = 0.2
        y_1 = math_functions.langmuir_isotherm(c_e, q_m, K_s)

        phi_h2o = 0.8
        y_2 = math_functions.langmuir_isotherm_hydrogel(c_e, q_m, K_s, phi_h2o)

        c_e_2 = [5, 6, 7, 8]
        K_s_2 = 0.8
        y_3 = math_functions.langmuir_comp(c_e, c_e_2, q_m, K_s, K_s_2)

        x = list(range(50))
        y_4 = math_functions.triangle(x, 22.3, 44.2, 33.1, 12.05, 3)
        y_5 = math_functions.gaussian(
            x, [1.2, 5], [3, 33.5], [1.1, 1], [4, 0.3])
        y_6 = math_functions.boxcar(x, 12.1, 15.3)
        y_7 = math_functions.boxcar_convolution(
            x, 12.3, 35.25, math_functions.gaussian, [1, 8.1, 0, 2],
            y_offset=1.1)
        y_8 = math_functions.piecewise_polynomial(
            x, [[3.2, 4.1, 5.1], [2, 4.1]], 33.2)
        y_9 = math_functions.piecewise_polynomial(
            x, [3.2, 4.1, 5.1], [])
        y_10 = math_functions.flory_rehner(
            [0.1, 0.2], 7000.3, 0.2, 0.4)
        y_11 = math_functions.Herschel_Bulkley(x, 200, 3, 0.6)
        y_12 = math_functions.cum_dist_normal(x, 3, 12.1, amp=3.2)
        y_13 = math_functions.cum_dist_normal_with_rise(
            x, 3.2, 25.8, 2.01, amp=1.1, linear_rise='full')
        y_14 = math_functions.cum_dist_normal_with_rise(
            x, 3.2, 25.8, 2.01, amp=1.1, linear_rise='left')
        y_15 = math_functions.cum_dist_normal_with_rise(
            x, 3.2, 25.8, 2.01, amp=1.1, linear_rise='right')
